from cache import Cache
import data
import nltk
import pandas
import results
import sys

def frequenciesLoader(articles, n, domain):
    texts = data.domain(articles, domain).contentWithoutClass
    state = {}
    for text in texts:
        ngrams = list(nltk.ngrams(text.split(), n))
        for k in ngrams:
            state[k] = 1+(state[k] if k in state else 0)
    return state

def frequenciesPath(inputHash, n):
    return lambda domain:\
        "frequencies/{inputHash}/{n}grams/{domain}.csv"\
                .format(inputHash=inputHash, n=n, domain=domain)

def loadFrequencies(f):
    csv = pandas.read_csv(f, sep='\t')
    return dict(zip(
            csv.ngram.map(lambda s: tuple(s.split(','))),
            csv.frequency
        ))

def saveFrequencies(data, f):
    pandas.DataFrame(data={
            'ngram': map(lambda t: ','.join(t), data.keys()),
            'frequency': data.values()
        }).to_csv(f, sep='\t', index=False)

def frequencies(source, n):
    return Cache(
            lambda domain: frequenciesLoader(source.articles, n, domain),
            pathPolicy=frequenciesPath(source.hash, n),
            serializer=saveFrequencies,
            unserializer=loadFrequencies
        )

def topLoader(frequencyEvaluator, n, ranks):
    return lambda domain:\
        dict(nltk.FreqDist(frequencyEvaluator(n, domain)).most_common(ranks))

def topPath(inputHash, n, ranks):
    return lambda domain:\
        "topNGrams/{inputHash}/{n}grams/top{ranks}/{domain}.csv".format(
                inputHash=inputHash,
                n=n,
                ranks=ranks,
                domain=domain
            )

def topNGrams(source, n, ranks):
    freq = frequencies(source, n)
    return Cache(
            topLoader(freq, n, ranks),
            pathPolicy=topPath(source.hash, n, ranks),
            serializer=saveFrequencies,
            unserializer=loadFrequencies
        )

def __syntax(this):
    print(
            "Syntax: {this} {required} {optional}".format(
                    this=this,
                    required="ARTICLES_DATA(.csv)",
                    optional="[NGRAM SIZE] [TOP_RANKS_SIZE] [DOMAIN]"
                ),
            file=sys.stderr
        )
    sys.exit(1)

def __compute(articlesSource, ns, ranksToTry, domains):
    for n in ns:
        for ranks in ranksToTry:
            cached = topNGrams(data.load(articlesSource), n, ranks)
            for domain in domains:
                cached(domain)

if __name__ == '__main__':
    argc = len(sys.argv)
    if argc < 2:
        __syntax(sys.argv[0])
    else:
        articlesSource = sys.argv[1]
        ns = [int(sys.argv[2])] if argc > 2 else range(1,4)
        ranksToTry = [int(sys.argv[3])] if argc > 3 else [10, 100, 50]
        domains = [sys.argv[4]] if argc > 4 else data.domains
        __compute(articlesSource, ns, ranksToTry, domains)
